<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\DB;
use App\User;
use Spatie\Permission\Models\Role;
use Spatie\Permission\Models\Permission;
use DataTables,Auth;

class UserController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {
        return view('users');
    }

    public function getUserList(Request $request)
    {
        
        $data  = User::get();
        
        return Datatables::of($data)
                ->addColumn('roles', function($data){
                    $roles = $data->getRoleNames()->toArray();
                    $badge = '';
                    if($roles){
                        $badge = implode(' , ', $roles);
                    }

                    return $badge;
                })
                ->addColumn('permissions', function($data){
                    $roles = $data->getAllPermissions();
                    $badges = '';
                    foreach ($roles as $key => $role) {
                        $badges .= '<span class="badge badge-dark m-1">'.$role->name.'</span>';
                    }

                    return $badges;
                })
                ->addColumn('action', function($data){
                    if($data->name == 'Super Admin'){
                        return '';
                    }
                    if (Auth::user()->can('manage_user')){
                        return '<div class="table-actions">
                                <a href="'.url('user/'.$data->id).'" ><i class="ik ik-edit-2 f-16 mr-15 text-green"></i></a>
                                <a href="'.url('user/delete/'.$data->id).'"><i class="ik ik-trash-2 f-16 text-red"></i></a>
                            </div>';
                    }else{
                        return '';
                    }
                })
                ->rawColumns(['roles','permissions','action'])
                ->make(true);
    }

    public function create()
    {
        try
        {
            $roles = Role::pluck('name','id');
            return view('create-user', compact('roles'));

        }catch (\Exception $e) {
            $bug = $e->getMessage();
            return redirect()->back()->with('error', $bug);

        }
    }

    public function store(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'name' => 'required | string ',
            'email' => 'required | email | unique:users',
            'password' => 'required | confirmed',
            'dist_code' => 'required',
            'state_code' => 'required',
            'emp_code' => 'required',
            'role' => 'required'
        ]);
        
        if($validator->fails()) {
            return redirect()->back()->withInput()->with('error', $validator->messages()->first());
        }
        try
        {
            if($request->art_centre_code)
            {
                $art_centre_code = implode(', ',$request->art_centre_code);
            }
            else
            {
                $art_centre_code = $request->art_centre_code;
            }
            // store user information
            $user = User::create([
                        'name' => $request->name,
                        'email' => $request->email,
                        'emp_code' => $request->emp_code,
                        'dist_code' => $request->dist_code,
                        'state_code' => $request->state_code,
                        'art_centre_code' => $art_centre_code,
                        'password' => Hash::make($request->password),
                    ]);

            // assign new role to the user
            $user->syncRoles($request->role);
            
            $data = array(
                "user_id"=>$user['id'],
                "client_reg_add"=>$request->client_reg_add,
                "client_reg_view"=>$request->client_reg_view,
                "client_reg_modify"=>$request->client_reg_modify,
                "client_reg_delete"=>$request->client_reg_delete,
                "client_map_add"=>$request->client_map_add,
                "client_map_view"=>$request->client_map_view,
                "client_map_modify"=>$request->client_map_modify,
                "client_map_delete"=>$request->client_map_delete,
                "outreach_add"=>$request->outreach_add,
                "outreach_view"=>$request->outreach_view,
                "outreach_modify"=>$request->outreach_modify,
                "outreach_delete"=>$request->outreach_delete,
                "calling_add"=>$request->calling_add,
                "calling_view"=>$request->calling_view,
                "calling_modify"=>$request->calling_modify,
                "calling_delete"=>$request->calling_delete,
                "eac_add"=>$request->eac_add,
                "eac_view"=>$request->eac_view,
                "eac_modify"=>$request->eac_modify,
                "eac_delete"=>$request->eac_delete,
                "doe_add"=>$request->doe_add,
                "doe_view"=>$request->doe_view,
                "doe_modify"=>$request->doe_modify,
                "doe_delete"=>$request->doe_delete,
                "report_view"=>$request->report_view,
            );
        
            $insdata = DB::table('users_permissions')->insert($data);
            if($user){ 
                return redirect('users')->with('success', 'New user created!');
            }else{
                return redirect('users')->with('error', 'Failed to create new user! Try again.');
            }
        }catch (\Exception $e) {
            return $e;
            $bug = $e->getMessage();
            return redirect()->back()->with('error', $bug);
        }
    }

    public function edit($id)
    {
        try
        {
            $user  = User::with('roles','permissions')->find($id);
            $user_permission  = DB::table('users_permissions')->where('user_id',$id)->get();
            if($user){
                $user_role = $user->roles->first();
                $roles     = Role::pluck('name','id');

                return view('user-edit', compact('user','user_role','roles','user_permission'));
            }else{
                return redirect('404');
            }

        }catch (\Exception $e) {
            $bug = $e->getMessage();
            return redirect()->back()->with('error', $bug);
        }
    }
    
    public function edit_user_profile(Request $request)
    {
        $user  = User::find($request->id);
        
        if($request->hasfile('profile'))
        {
            $file = $request->profile;
            $fileNameWithExt = $file->getClientOriginalName();
            $filename = pathinfo($fileNameWithExt, PATHINFO_FILENAME);
            $extension = $file->getClientOriginalExtension();
            $file_name = $filename.'_'.time().'.'.$extension;
            $file->storeAs('public/profile/',$file_name);
            $user->profile=$file_name;
            $user->save();
        }
        return redirect()->back()->with('success', 'User profile updated succesfully!');     
    }

    public function update(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id'       => 'required',
            'name'     => 'required | string ',
            'email'    => 'required | email',
            'role'     => 'required'
        ]);

        // check validation for password match
        if(isset($request->password)){
            $validator = Validator::make($request->all(), [
                'password' => 'required | confirmed'
            ]);
        }
        
        if ($validator->fails()) {
            return redirect()->back()->withInput()->with('error', $validator->messages()->first());
        }

        try
        {
            $user = User::find($request->id);
            if($request->art_centre_code)
            {
                $art_centre_code = implode(', ',$request->art_centre_code);
            }
            else
            {
                $art_centre_code = $request->art_centre_code;
            }
            $update = $user->update([
                'name' => $request->name,
                'email' => $request->email,
                'emp_code' => $request->emp_code,
                'dist_code' => $request->dist_code,
                'state_code' => $request->state_code,
                'art_centre_code' => $art_centre_code,
            ]);
            
            $user->syncRoles($request->role);
            
            $data = array(
                "user_id"=>$user['id'],
                "client_reg_add"=>$request->client_reg_add,
                "client_reg_view"=>$request->client_reg_view,
                "client_reg_modify"=>$request->client_reg_modify,
                "client_reg_delete"=>$request->client_reg_delete,
                "client_map_add"=>$request->client_map_add,
                "client_map_view"=>$request->client_map_view,
                "client_map_modify"=>$request->client_map_modify,
                "client_map_delete"=>$request->client_map_delete,
                "outreach_add"=>$request->outreach_add,
                "outreach_view"=>$request->outreach_view,
                "outreach_modify"=>$request->outreach_modify,
                "outreach_delete"=>$request->outreach_delete,
                "calling_add"=>$request->calling_add,
                "calling_view"=>$request->calling_view,
                "calling_modify"=>$request->calling_modify,
                "calling_delete"=>$request->calling_delete,
                "eac_add"=>$request->eac_add,
                "eac_view"=>$request->eac_view,
                "eac_modify"=>$request->eac_modify,
                "eac_delete"=>$request->eac_delete,
                "doe_add"=>$request->doe_add,
                "doe_view"=>$request->doe_view,
                "doe_modify"=>$request->doe_modify,
                "doe_delete"=>$request->doe_delete,
                "report_view"=>$request->report_view,
            );

            $insdata = DB::table('users_permissions')->where('user_id',$request->id)->update($data);
            // update password if user input a new password
            if(isset($request->password)){
                $update = $user->update([
                    'password' => Hash::make($request->password)
                ]);
            }

            // sync user role
            $user->syncRoles($request->role);
            return redirect('users')->with('success', 'User information updated succesfully!');
        }
        catch (\Exception $e) 
        {
            return $e->getMessage();
            $bug = $e->getMessage();
            return redirect()->back()->with('error', $bug);
        }
    }

    public function delete($id)
    {
        $user   = User::find($id);
        if($user){
            $user->delete();
            return redirect('users')->with('success', 'User removed!');
        }else{
            return redirect('users')->with('error', 'User not found');
        }
    }

    public function getdistrict(Request $request)
    {
        $showdata = DB::table('district_master')->where('state_code',$request->state_code)->get();
        $html = "<option value=''>Select District Code</option>";
        if(count($showdata) > 0)
        {
            foreach($showdata as $data)
            {
                $html .= "<option value='".$data->district_code."'>".$data->district_code."</option>";
            }
        }

        $role = DB::table('roles')->where('id',$request->role)->first();

        if($role->name == "Super Admin" || $role->name == "Admin" || $role->name == "National_Manager")
        {
            $showart = DB::table('art_centre_master')->get();
        }
        else
        {
            $showart = DB::table('art_centre_master')->where('state_code',$request->state_code)->get();
        }
        $arthtml = "<option value=''>Select ART Center</option>";
        if(count($showart) > 0)
        {
            foreach($showart as $data)
            {
                $arthtml .= "<option value='".$data->art_centre_code."'>".$data->art_centre_code."</option>";
            }
        }
        
        return json_encode(['html' => $html, 'arthtml' => $arthtml]);
    }

    public function getartcenter(Request $request)
    {
        $role = DB::table('roles')->where('id',$request->role)->first();
        if($role->name == "Super Admin" || $role->name == "Admin" || $role->name == "National_Manager")
        {
            $showdata = DB::table('art_centre_master')->get();
        }
        else if($role->name == "State_Manager")
        {
            $showdata = DB::table('art_centre_master')->where('state_code',$request->state_code)->get();
        }
        else
        {
            $showdata = DB::table('art_centre_master')->where('dist_code',$request->dist_code)->get();
        }
        
        $html = "<option value=''>Select ART Center</option>";
        if(count($showdata) > 0)
        {
            foreach($showdata as $data)
            {
                $html .= "<option value='".$data->art_centre_code."'>".$data->art_centre_code."</option>";
            }
        }
        return json_encode(['html' => $html]);
    }
}